@@ -2,18 +2,20 @@
 
 #    This file is part of python-registry.
 #
-#    python-registry is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#   Copyright 2011 Will Ballenthin <willi.ballenthin@mandiant.com>
+#                    while at Mandiant <http://www.mandiant.com>
 #
-#    python-registry is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
 #
-#    You should have received a copy of the GNU General Public License
-#    along with python-registry.  If not, see <http://www.gnu.org/licenses/>.
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
 
 import sys
 import RegistryParse
@@ -189,7 +191,7 @@ class RegistryKey(object):
         # a new RegistryKey from the NKRecord parent key, rather
         # than using the parent of this instance, if it exists.
         try:
-            return self._nkrecord.parent_key()
+            return RegistryKey(self._nkrecord.parent_key())
         except RegistryParse.ParseException:
             raise RegistryKeyHasNoParentException(self.name())
 
@@ -255,16 +257,19 @@ class Registry(object):
     """
     A class for parsing and reading from a Windows Registry file.
     """    
-    def __init__(self, filename):
+    def __init__(self, filelikeobject):
         """
         Constructor.
         Arguments:
-        - `filename`: A string containing the filename of the Windows Registry file, such as
-        NTUSER.DAT.
+        - `filelikeobject`: A file-like object with a .read() method.  
+              If a Python string is passed, it is interpreted as a filename, 
+              and the corresponding file is opened.
         """
-        with open(filename) as f:
-            self._buf = f.read()
-
+        try:
+            self._buf = filelikeobject.read()
+        except AttributeError:
+            with open(filelikeobject, "rb") as f:
+                self._buf = f.read()
         self._regf = RegistryParse.REGFBlock(self._buf, 0, False)
 
     def root(self):
