@@ -2,18 +2,20 @@
 
 #    This file is part of python-registry.
 #
-#    python-registry is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#   Copyright 2011 Will Ballenthin <willi.ballenthin@mandiant.com>
+#                    while at Mandiant <http://www.mandiant.com>
 #
-#    python-registry is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
 #
-#    You should have received a copy of the GNU General Public License
-#    along with python-registry.  If not, see <http://www.gnu.org/licenses/>.
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
 
 import struct
 from datetime import datetime
@@ -320,7 +322,7 @@ class HBINCell(RegistryBlock):
 
     def offset(self):
         """
-        Accessor for absolute offset of this block.
+        Accessor for absolute offset of this HBINCell.
         """
         return self._offset
 
@@ -330,6 +332,12 @@ class HBINCell(RegistryBlock):
         """
         return self._offset + 0x4
 
+    def raw_data(self):
+        """
+        Get the raw data from the buffer contained by this HBINCell.
+        """
+        return self._buf[self.data_offset():self.data_offset() + self.size()]
+
     def data_id(self):
         """
         Get the ID string of the data block of this HBINCell.
@@ -348,6 +356,11 @@ class HBINCell(RegistryBlock):
         return h.first_hbin().offset() + offset
 
     def child(self):
+        """
+        Make a _guess_ as to the contents of this structure and
+        return an instance of that class, or just a DataRecord
+        otherwise.
+        """
         if self.is_free():
             raise RegistryStructureDoesNotExist("HBINCell is free at 0x%x" % (self.offset()))
 
@@ -367,6 +380,8 @@ class HBINCell(RegistryBlock):
             return RIRecord(self._buf, self.data_offset(), self)
         elif id_ == "sk":
             return SKRecord(self._buf, self.data_offset(), self)
+        elif id_ == "db":
+            return DBRecord(self._buf, self.data_offset(), self)
         else:
             return DataRecord(self._buf, self.data_offset(), self)
 
@@ -414,12 +429,77 @@ class DataRecord(Record):
 
     def __str__(self):
         return "Data Record at 0x%x" % (self.offset())
+
+class DBIndirectBlock(Record):
+    """ 
+    The DBIndirect block is a list of offsets to DataRecords with data 
+    size up to 0x3fd8.
+    """
+    def __init__(self, buf, offset, parent):
+        """
+        Constructor.
+        Arguments:
+        - `buf`: Byte string containing Windows Registry file.
+        - `offset`: The offset into the buffer at which the block starts.
+        - `parent`: The parent block, which links to this block. This should be an HBINCell.
+        """
+        super(DBIndirectBlock, self).__init__(buf, offset, parent)
+
+    def __str__(self):
+        return "Large Data Block at 0x%x" % (self.offset())
         
+    def large_data(self, length):
+        """
+        Get the data pointed to by the indirect block. It may be large.
+        Return a byte array.
+        """
+        b = bytearray()
+        count = 0
+        while length > 0:
+            off = self.abs_offset_from_hbin_offset(self.unpack_dword(4 * count))
+            size = min(0x3fd8, length)
+            b += HBINCell(self._buf, off, self).raw_data()[0:size]
+
+            count += 1
+            length -= size
+        return b
+
+class DBRecord(Record):
+    """
+    A DBRecord is a large data block, which is not thoroughly documented.
+    Its similar to an inode in the Ext file systems.
+    """
+    def __init__(self, buf, offset, parent):
+        """
+        Constructor.
+        Arguments:
+        - `buf`: Byte string containing Windows Registry file.
+        - `offset`: The offset into the buffer at which the block starts.
+        - `parent`: The parent block, which links to this block. This should be an HBINCell.
+        """
+        super(DBRecord, self).__init__(buf, offset, parent)
+
+        _id = self.unpack_string(0x0, 2)
+        if _id != "db":
+            raise ParseException("Invalid DB Record ID")
+
+    def __str__(self):
+        return "Large Data Block at 0x%x" % (self.offset())
+        
+    def large_data(self, length):
+        """
+        Get the data described by the DBRecord. It may be large.
+        Return a byte array.
+        """
+        off = self.abs_offset_from_hbin_offset(self.unpack_dword(0x4))
+        cell = HBINCell(self._buf, off, self)
+        dbi = DBIndirectBlock(self._buf, cell.data_offset(), cell)
+        return dbi.large_data(length)
+
 class VKRecord(Record):
     """
     The VKRecord holds one name-value pair.  The data may be one many types, including
-    strings, integers, and binary data.
-    """
+    strings, integers, and binary data.    """
     def __init__(self, buf, offset, parent):
         """
         Constructor.
@@ -579,18 +659,21 @@ class VKRecord(Record):
             if data_length >= 0x80000000:
                 # data is contained in the data_offset field
                 s = struct.unpack_from("<%ds" % (4), self._buf, data_offset)[0]
+            elif 0x3fd8 < data_length < 0x80000000:
+                db = HBINCell(self._buf, data_offset, self).child() # should be db
+                s = db.large_data(data_length)
             else:
                 d = HBINCell(self._buf, data_offset, self)
                 s = struct.unpack_from("<%ds" % (data_length), self._buf, d.data_offset())[0]
 
             try:
-                s = s.decode("utf16").encode("utf8")
-            except UnicodeDecodeError:
+                s = s.decode("utf16").encode("utf8").decode("utf8") # iron out the kinks by 
+            except UnicodeDecodeError:                              # converting to and back to a Python str
                 try:
-                    s = s.decode("utf8").encode("utf8")
+                    s = s.decode("utf8").encode("utf8").decode("utf8")
                 except UnicodeDecodeError:
                     try:
-                        s = s.decode("utf8", "replace").encode("utf8")
+                        s = s.decode("utf8", "replace").encode("utf8").decode("utf8")
                     except:
                         print "Well at this point you are screwed."
                         raise
@@ -600,6 +683,9 @@ class VKRecord(Record):
             if data_length >= 0x80000000:
                 data_length -= 0x80000000
                 return self._buf[data_offset:data_offset + data_length]
+            elif 0x3fd8 < data_length < 0x80000000:
+                db = HBINCell(self._buf, data_offset, self).child() # should be db
+                return db.large_data(data_length)
             return self._buf[data_offset + 4:data_offset + 4 + data_length]
         elif data_type == RegDWord:
             return self.unpack_dword(0x8)
@@ -608,6 +694,9 @@ class VKRecord(Record):
                 # this means data_length < 5, so it must be 4, and
                 # be composed of completely \x00, so the strings are empty
                 return []
+            elif 0x3fd8 < data_length < 0x80000000:
+                db = HBINCell(self._buf, data_offset, self).child() # should be db
+                data = db.large_data(data_length).decode("utf16")
             else:
                 data = self._buf[data_offset + 4:data_offset + 4 + data_length].decode("utf16")
             return data.split("\x00")
@@ -619,6 +708,9 @@ class VKRecord(Record):
             if data_length >= 0x80000000:
                 data_length -= 0x80000000
                 return self._buf[data_offset:data_offset + data_length]
+            elif 0x3fd8 < data_length < 0x80000000:
+                db = HBINCell(self._buf, data_offset, self).child() # should be db
+                return db.large_data(data_length)
             return self._buf[data_offset + 4:data_offset + 4 + data_length]
         elif data_type == RegBigEndian:
             print "Warning: Data type RegBigEndian not yet supported"
@@ -726,8 +818,7 @@ class SubkeyList(Record):
 
 class RIRecord(SubkeyList):
     """
-    """
-    
+    """    
     def __init__(self, buf, offset, parent):
         """
         
@@ -864,7 +955,7 @@ class NKRecord(Record):
         _id = self.unpack_string(0x0, 2)
         if _id != "nk":
             raise ParseException("Invalid NK Record ID")
-        
+
     def __str__(self):
         classname = self.classname()
         if not self.has_classname():
@@ -1102,3 +1193,13 @@ class HBINBlock(RegistryBlock):
             except RegistryStructureDoesNotExist:
                 break
 
+
+
+
+
+
+
+
+
+
+
